/*
 * This file is part of Odin.
 *
 * Copyright (c) 2011 CraftFire <http://www.craftfire.com/>
 * Odin is licensed under the GNU Lesser General Public License.
 *
 * Odin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Odin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.craftfire.odin.layer.bukkit.api.events.plugin;

import org.bukkit.entity.Player;
import org.bukkit.event.Cancellable;
import org.bukkit.event.HandlerList;

import com.craftfire.odin.layer.bukkit.api.events.player.OdinPlayerEvent;
import com.craftfire.odin.layer.bukkit.managers.OdinPlayer;

public class OdinMessageEvent extends OdinPlayerEvent implements Cancellable {
    private static final HandlerList handlers = new HandlerList();
    private String message;
    private boolean cancel;

    public OdinMessageEvent(OdinPlayer receiver, String message) {
        super(receiver);
        this.message = message;
        this.cancel = false;
    }

    /**
     * Returns the message
     *
     * @return message of the event
     */
    public String getMessage() {
        return this.message;
    }

    /**
     * Sets the message
     *
     * @param message the message
     */
    public void setMessage(String message) {
        this.message = message;
    }

    @Override
    public boolean isCancelled() {
        return this.cancel;
    }

    @Override
    public void setCancelled(boolean cancel) {
        this.cancel = cancel;
    }

    @Override
    public HandlerList getHandlers() {
        return handlers;
    }

    public static HandlerList getHandlerList() {
        return handlers;
    }
}
